
/*
 * Defines Flickr Photo Object
 * 
 * App Retrieves recent photos from Flickr Stream
 * Displays a grid of photos on your Android Phone or Tablet
 * Uses the AnyMote protocol to "Fling" a selected picture to Google TV
 * 
 * To use: Get Your own API key from Flickr and replace the API_KEY value below
 * 
 * Carmen Delessio
 * Author "Sams Teach Yourself Google TV App Development in 24 Hours"
 * 
 * http://www.amazon.com/Teach-Yourself-Google-Development-Hours/dp/0672336030/
 */

package com.bffmedia.flickrongtv;
import java.util.ArrayList;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;


public class FlickrPhoto extends Object{
	String id;
	String owner;
	String secret;
	String server;
	String farm;
	String title;
	Boolean isPublic;
	Boolean isFriend;
	Boolean isFamily;


	public FlickrPhoto(JSONObject jsonPhoto) throws JSONException{	
		this.id=(String) jsonPhoto.optString("id"); 	
		this.secret=(String) jsonPhoto.optString("secret"); 
		this.owner=(String) jsonPhoto.optString("owner");
		this.server=(String) jsonPhoto.optString("server");
		this.farm=(String) jsonPhoto.optString("farm");
		this.title=(String) jsonPhoto.optString("title");

		this.isPublic=(Boolean) jsonPhoto.optBoolean("ispublic");
		this.isFriend=(Boolean) jsonPhoto.optBoolean("isfriend");
		this.isFamily=(Boolean) jsonPhoto.optBoolean("isfamily");


	}
	public String getPhotoUrl(Boolean big){
	    String opt = "n";
	    if (big)
	    	opt ="c";
		String photoUri = 
		"http://farm"+this.farm + ".staticflickr.com/"+this.server+"/"+ this.id+"_"+this.secret +"_" + opt +".jpg";
		return photoUri;		
	}


	public static ArrayList <FlickrPhoto> makePhotoList (String photoData ) throws JSONException, NullPointerException {
		ArrayList <FlickrPhoto> flickrPhotos = new ArrayList<FlickrPhoto>();		

		JSONObject data  = new JSONObject(photoData);
		JSONObject photos = data.optJSONObject("photos");
		JSONArray photoArray = photos.optJSONArray("photo");
		for(int i = 0; i < photoArray.length(); i++) {
			JSONObject photo=	(JSONObject) photoArray.get(i);
			FlickrPhoto currentPhoto = new FlickrPhoto (photo);
			flickrPhotos.add(currentPhoto);

		}

		return flickrPhotos;
	}


}
