
/*
 * Retrieves recent photos from Flickr Stream
 * Displays a grid of photos on your Android Phone or Tablet
 * Uses the AnyMote protocol to "Fling" a selected picture to Google TV
 * 
 * To use: Get Your own API key from Flickr and replace the API_KEY value below
 * 
 * Carmen Delessio
 * Author "Sams Teach Yourself Google TV App Development in 24 Hours"
 * 
 * http://www.amazon.com/Teach-Yourself-Google-Development-Hours/dp/0672336030/
 */

package com.bffmedia.flickrongtv;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Date;

import org.json.JSONException;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.BaseAdapter;
import android.widget.Button;
import android.widget.GridView;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.Toast;

import com.example.google.tv.anymotelibrary.client.AnymoteClientService;
import com.example.google.tv.anymotelibrary.client.AnymoteClientService.ClientListener;
import com.example.google.tv.anymotelibrary.client.AnymoteSender;





public class RecentPhotoActivity extends Activity implements ClientListener {
	public final static String API_KEY ="--PUT YOUR API KEY HERE--";
	public final static String NUM_PHOTOS ="12";
	
	private AnymoteClientService mAnymoteClientService;
	private ProgressBar progressBar;
	private Handler handler;
	private AnymoteSender anymoteSender;   
	private ArrayList<FlickrPhoto> mPhotos = new ArrayList<FlickrPhoto>();
	private GridView mGridView;



	private ServiceConnection mConnection = new ServiceConnection() {
		public void onServiceConnected(ComponentName name, IBinder service) {
			mAnymoteClientService = ((AnymoteClientService.AnymoteClientServiceBinder) service)
					.getService();
			mAnymoteClientService.attachClientListener(RecentPhotoActivity.this);
		}

		public void onServiceDisconnected(ComponentName name) {
			mAnymoteClientService.detachClientListener(RecentPhotoActivity.this);
			mAnymoteClientService = null;
		}
	};
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);
		progressBar = (ProgressBar) findViewById(R.id.a_progressbar);
		progressBar.setVisibility(View.VISIBLE);

		Button go = (Button) findViewById(R.id.go);
		go.setOnClickListener(new OnClickListener() {
			@Override
			public void onClick(View v) {
				LoadPhotos task = new LoadPhotos();
				task.execute();
			}
		});

		mGridView = (GridView)findViewById(R.id.gridView1);
		mGridView.setOnItemClickListener(new OnItemClickListener() {
			public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
				String selectedPhoto = (String) view.getTag();  // we set tag in the adapter
				final Intent intent = new Intent(Intent.ACTION_VIEW, Uri.parse(selectedPhoto)); // make it a URI and intent 
				anymoteSender.sendUrl (intent.toUri(Intent.URI_INTENT_SCHEME)); // send it to the TV!
			}		
		});

		handler = new Handler();
		Intent intent = new Intent(RecentPhotoActivity.this, AnymoteClientService.class);
		bindService(intent, mConnection, Context.BIND_AUTO_CREATE);
	}


	@Override
	public void onConnected(final AnymoteSender anymoteSender) {
		this.anymoteSender = anymoteSender;
		handler.post(new Runnable() {
			public void run() {
				progressBar.setVisibility(View.INVISIBLE);
				LoadPhotos task = new LoadPhotos(); // LoadPhotos after we are connected to a TV
				task.execute();
			}
		});
	}

	@Override
	public void onDisconnected() {
		Toast.makeText(
				RecentPhotoActivity.this, R.string.pairing_failed_toast, Toast.LENGTH_LONG)
				.show();
		this.anymoteSender = null;
	}


	@Override
	public void onConnectionFailed() {
		System.out.println("connection failed");
	}
	@Override
	protected void onDestroy() {
		if (mAnymoteClientService != null) {
			mAnymoteClientService.detachClientListener(this);
		}
		unbindService(mConnection);
		super.onDestroy();
	}



	private class LoadPhotos extends AsyncTask<String , String , Long > {


		public LoadPhotos (){

		}
		@Override
		protected void onPreExecute() {
		}

		@Override
		protected void onPostExecute(Long result) {
			mGridView.setAdapter(new ImageAdapter(RecentPhotoActivity.this.getApplicationContext()));
		}

		@Override
		protected Long doInBackground(String... params) {

			HttpURLConnection connection = null;
			try {
				URL dataUrl = new URL("http://api.flickr.com/services/rest/?method=flickr.photos.getRecent&api_key="+ API_KEY+ "&per_page=" + NUM_PHOTOS +"&format=json&nojsoncallback=1");
				connection = (HttpURLConnection) dataUrl.openConnection();
				connection.connect();
				InputStream is = connection.getInputStream();
				BufferedReader reader = new BufferedReader(new InputStreamReader(is));
				String responseString;
				StringBuilder sb = new StringBuilder();
				while ((responseString = reader.readLine()) != null) {
					sb = sb.append(responseString);
				}
				String photoData = sb.toString();
				mPhotos = FlickrPhoto.makePhotoList(photoData);

				return (0l);
			} catch (MalformedURLException e) {
				e.printStackTrace();
				return (1l);
			} 
			catch (IOException e) {
				System.out.println("IOException");
				e.printStackTrace();
				return (1l);
			} catch (NullPointerException e) {

				e.printStackTrace();
				return (1l);
			} catch (JSONException e) {
				e.printStackTrace();
				return (1l);
			} finally {
				connection.disconnect();
			}
		}
	}

	public class ImageAdapter extends BaseAdapter {
		private Context mContext;

		public ImageAdapter(Context c) {
			mContext = c;
		}

		public int getCount() {
			return mPhotos.size();
		}

		public Object getItem(int position) {
			return null;
		}

		public long getItemId(int position) {
			return 0;
		}

		// create a new ImageView for each item referenced by the Adapter
		public View getView(int position, View convertView, ViewGroup parent) {
			ImageView imageView;
			if (convertView == null) {  // if it's not recycled, initialize some attributes
				imageView = new ImageView(mContext);
				imageView.setLayoutParams(new GridView.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT));
				imageView.setScaleType(ImageView.ScaleType.CENTER_CROP);
				imageView.setPadding(8, 8, 8, 8);
			} else {
				imageView = (ImageView) convertView;
			}

			FlickrPhoto currentPhoto = (FlickrPhoto) (mPhotos.get(position));
			imageView.setTag(currentPhoto.getPhotoUrl(true));
			LoadImage loadImage = new LoadImage(imageView, currentPhoto.getPhotoUrl(false));
			loadImage.execute();
			return imageView;
		}


	}

	private class LoadImage extends AsyncTask<String , String , Long > {
		ImageView mImageView;
		String  mImageString;
		Bitmap mBitmap;

		public LoadImage (ImageView v, String imageString){
			mImageView =v;
			mImageString = imageString;
		}
		@Override
		protected void onPreExecute() {
		}

		@Override
		protected void onPostExecute(Long result) {
			if (result==0){
				mImageView.setImageBitmap(mBitmap);
				Date now = new Date();
				System.out.println("time end: " + now.getTime() );
			}
		}
		@Override
		protected Long doInBackground(String... params) {
			URL imageUrl;
			try {
				imageUrl = new URL(mImageString);

				URLConnection connection = imageUrl.openConnection();
				connection.connect();
				InputStream is = connection.getInputStream();
				mBitmap = BitmapFactory.decodeStream(is);
				return 0l;
			} catch (MalformedURLException e) {
				e.printStackTrace();
				return 1l;
			} catch (IOException e) {
				e.printStackTrace();
				return 1l;
			}
		}

	}


}
